/**
 * Autor:     Alberto Gil Tesa
 * Web:       http://giltesa.com
 * Licencia:  CC BY-NC-SA 3.0
 *
 * ATTiny85 pinout:
 *                         _____
 *                        |1   8| VCC
 *                        |2   7|
 *    Potenciometro -> A2 |3   6| D1 <- Tira de leds
 *                    GND |4___5| D0 <- Sensor PIR
*/


// PINES
#define LED 1     // Digital PWM  - Tira de leds
#define PIR 0     // Digital      - Sensor movimiento
#define POT A2    // Analogico    - Regulador brillo

// BRILLO:
#define bMinimo   3    // Valor minimo para que el led este encendido.
#define bMargen   4    // Diferencia que ha de haber para que se considere que se ha cambiado el brillo.
byte bAnterior  = 0;
byte bActual    = 0;
byte bAtenuado  = 0;

// MODO AHORRO:
#define tiempoApagado      600000   // Tiempo para entrar en modo ahorro (apagado).
#define tiempoAtenuado     18       // Velocidad que se tarda en aumentar o disminuir cada nivel de brillo.
unsigned long ultimoUso  = 0;       // El momento en el que se utilizo por ultima vez el potenciometro o se detecto al usuario.
boolean enAhorro         = false;   // Indica si los Leds se apagaron por inactividad del usuario.
boolean cambiandoBrillo  = false;   // Indica que se esta cambiando el brillo (aumentandolo o decrementandolo gradualmente)



/**
 * Funcion inicial de configuracion.
 */
void setup()
{
    pinMode(LED, OUTPUT);

    // Poniendo el programa en ahorro y las variables de brillo con el mismo valor nos aseguramos
    // de que al reiniciarse el microcontrolador, por un corte de luz, las luces permanezcan apagadas.
    enAhorro    = true;
    bAnterior   = getBrillo();
    bActual     = bAnterior;
}



/**
 * Funcion principal.
 */
void loop()
{
    bActual = getBrillo();


    // Se comprueba si el potenciometro ha sido movido. Segun el estado del programa:
    if( getDiferenciaBrillo() > bMargen )
    {
        bAnterior = bActual;
        ultimoUso = millis();

        // Comienza la desactivacion del modo ahorro, los leds se endenderan gradualmente:
        if( enAhorro )
        {
            setBrillo(true);
        }
        // Los leds cambiaran el brillo por el nuevo valor, siempre que el nuevo valor supere el minimo para que estos se enciendan:
        else if( bActual > bMinimo )
        {
            analogWrite( LED, bActual );
        }
        // Si no se supera el minimo los leds se apagan:
        else
        {
            analogWrite( LED, 0 );
        }
    }
    // Prosigue la desactivacion del modo ahorro:
    else if( enAhorro && cambiandoBrillo )
    {
        setBrillo(true);
    }


    // Registra la ultima vez que se detecto la presencia del usuario:
    if( digitalRead(PIR) )
    {
        ultimoUso = millis();

        // Si se estaba activando el modo ahorro, se desactiva gradualmente:
        if( !enAhorro && cambiandoBrillo )
            setBrillo(true);
    }


    // Activa el modo ahorro si el usuario no ha interactuado con los sensores en el tiempoApagado especificado:
    if( !enAhorro  &&  millis()-ultimoUso > tiempoApagado )
    {
        setBrillo(false);
    }

}



/**
 * Enciende o apaga el LED con efecto gradual.
 */
void setBrillo( boolean estado )
{

    if( !cambiandoBrillo )
    {
        bAtenuado = estado ? 0 : bActual;
        cambiandoBrillo = true;
    }


    // Sube el brillo:
    if( estado )
    {
        if( bAtenuado < bActual)
        {
            analogWrite( LED, ++bAtenuado );
            delay(tiempoAtenuado);
        }
        else
        {
            enAhorro = false;
            cambiandoBrillo = false;
            bAnterior = bActual;
        }
    }
    // Baja el brillo:
    else
    {
        if( bAtenuado > 0 )
        {
            analogWrite( LED, --bAtenuado );
            delay(tiempoAtenuado);
        }
        else
        {
            enAhorro = true;
            cambiandoBrillo = false;
            bAnterior = bActual;
        }
    }


}



/**
 * Lee el valor del potenciometro y devuelve
 * el brillo correspondiente para el led.
 */
byte getBrillo()
{
    return map(analogRead(POT), 0, 1023, 0, 255);
}



/**
 * Devuelve la diferencia de brillo entre la medicion
 * anterior y la actual.
 */
byte getDiferenciaBrillo()
{
    return bAnterior > bActual ? bAnterior-bActual : bActual-bAnterior;
}