/**
 * Autor:     Alberto Gil Tesa
 * Web:       http://giltesa.com
 * Licencia:  CC BY-NC-SA 3.0
 *
 * Arduino Mini Pro PinOut:
 *
 *  CONEXIONADO DE LOS COMPONENTES
 *                       ____________
 *                 PC   |TX0      RAW|
 *                 PC   |RX1      GND|
 *                 PC   |RST      RST|
 *                      |GND      VCC|
 *                      |2         A7|
 *        TIRA DE LED   |3~        A6|
 *  SENSOR MOVIMIENTO   |4         A5|
 *                      |5~        A4|
 *                      |6~        A3|
 *                      |7         A2|
 *                      |8         A1|   POTENCIOMETRO
 *                      |9~        A0|
 *                      |          13|
 *                      |          12|
 *                      |         ~11|
 *                      |_________~10|
 *
*/


// PINES
#define LED 3     // Digital PWM  - Tira de leds
#define PIR 4     // Digital      - Sensor movimiento
#define POT A1    // Analogico    - Regulador brillo

// BRILLO:
#define bMinimo   3    // Valor minimo para que el led este encendido.
#define bMargen   4    // Diferencia que ha de haber para que se considere que se ha cambiado el brillo.
byte bAnterior  = 0;
byte bActual    = 0;

// MODO AHORRO:
#define tiempoApagado      600000   // Tiempo para entrar en modo ahorro (apagado).
unsigned long ultimoUso  = 0;       // El momento en el que se utilizo por ultima vez el potenciometro o se detecto al usuario.
boolean enAhorro         = false;   // Indica si los Leds se apagaron por inactividad del usuario.



/**
 * Funcion inicial de configuracion.
 */
void setup()
{
    pinMode(LED, OUTPUT);

    // Poniendo el programa en ahorro y las variables de brillo con el mismo valor nos aseguramos
    // de que al reiniciarse el microcontrolador, por un corte de luz, las luces permanezcan apagadas.
    enAhorro    = true;
    bAnterior   = getBrillo();
    bActual     = bAnterior;
}



/**
 * Funcion principal.
 */
void loop()
{
    bActual = getBrillo();


    // Se comprueba si el potenciometro ha sido movido. Segun el estado del programa:
    if( getDiferenciaBrillo() > bMargen )
    {
        bAnterior = bActual;
        ultimoUso = millis();

        // Los leds cambiaran el brillo por el nuevo valor, siempre que el nuevo valor supere el minimo para que estos se enciendan:
        if( bActual > bMinimo || enAhorro )
        {
            enAhorro = false;
            analogWrite( LED, bActual );
        }
        // Si no se supera el minimo los leds se apagan:
        else
        {
            analogWrite( LED, 0 );
        }
    }


    // Registra la ultima vez que se detecto la presencia del usuario:
    if( digitalRead(PIR) )
    {
        ultimoUso = millis();
    }


    // Activa el modo ahorro si el usuario no ha interactuado con los sensores en el tiempoApagado especificado:
    if( !enAhorro  &&  millis()-ultimoUso > tiempoApagado )
    {
        enAhorro = true;
        analogWrite( LED, 0 );
    }

}



/**
 * Lee el valor del potenciometro y devuelve
 * el brillo correspondiente para el led.
 */
byte getBrillo()
{
    return map(analogRead(POT), 0, 1023, 0, 255);
}



/**
 * Devuelve la diferencia de brillo entre la medicion
 * anterior y la actual.
 */
byte getDiferenciaBrillo()
{
    return bAnterior > bActual ? bAnterior-bActual : bActual-bAnterior;
}